
# -------------------------------------------------------------------------
# DMvsVA PAPER 
# Functions
# -------------------------------------------------------------------------
#
# Author: EVANTHIA DIMARA
#
# -------------------------------------------------------------------------

library(boot)
library(PropCIs)
library(ggplot2)
library(simpleboot)


# Mean of observations
samplemean <- function(x, d) {
  return(mean(x[d]))
}

mergeStartEndColumnsToDuration <- function(dframe) 
{
  startTime <- strptime(dframe[,"startTime"], "%Y-%m-%d %H:%M:%S")
  endTime   <- strptime(dframe[,"endTime"], "%Y-%m-%d %H:%M:%S")
  duration  <- endTime - startTime 
  duration <- log(as.numeric(duration, units="secs"))  # 1) take the log of each individual time measurement 
  dframe <- dframe[ , -which(names(dframe) %in% c("startTime","endTime"))] # remove start - end time from the dframe
  cbind(dframe, duration)  # add duration column in the dframe
}


averagePerParticipant <- function(dframe, tech) 
{
  dataframe2 <- subset(dframe, isTraining==0 &  technique==tech)  # remove time of other techniques and trainings
  dataframe2
  dataframe2 <- aggregate(dataframe2[, "duration"], list(dataframe2$participantid, dataframe2$technique), mean) # 2) average the 4 repeated measures by participant
  dataframe2 <-  as.numeric(dataframe2[,"x"])  # fix the formating : string--> numeric
  return (dataframe2)
}

averagePerParticipantDM <- function(dframe, tech) 
{
  dataframe2 <- subset(dframe, technique==tech)  # remove time of other techniques and trainings
  dataframe2
  dataframe2 <- aggregate(dataframe2[, "duration"], list(dataframe2$participantid, dataframe2$technique), mean) # 2) average the 4 repeated measures by participant
  dataframe2 <-  as.numeric(dataframe2[,"x"])  # fix the formating : string--> numeric
  return (dataframe2)
}

# Mean of observations
samplemean <- function(x, d) {
  return(mean(x[d]))
}

diffMeanCI <- function(group1, group2) {
  pointEstimate <- samplemean(group1) - samplemean(group2)
  # Compute the bootstrap confidence interval
  set.seed(0) # make deterministic
  bootstrap_samples <- two.boot(sample1 = group1, sample2 = group2, FUN = samplemean, R = 5000)
  bootci <- boot.ci(bootstrap_samples, type = "bca")
  c(pointEstimate,  bootci$bca[4], bootci$bca[5])
}

#
# Function for computing the mean of non-normally distributed data and its
# 95% (bootstrap) confidence interval
#

meanCI <- function(observations) {
  samplemean <- function(x, d) {return(mean(x[d]))}
  mean <- samplemean(observations)
  # Compute the bootstrap confidence interval
  set.seed(0) # make deterministic
  bootstrap_samples <- boot(data = observations, statistic = samplemean, R = 5000)
  bootci <- boot.ci(bootstrap_samples, type = "bca")
  c(mean,  bootci$bca[4], bootci$bca[5])
}

#
# Function for computing the mean of normally distributed data and its
# exact 95% confidence interval
#
meanCI.normal <- function(observations) {
  samplemean <- mean(observations)
  ttest <- t.test(observations)
  lowerBound <- ttest[4]$conf.int[1]
  upperBound <- ttest[4]$conf.int[2]
  c(samplemean, lowerBound, upperBound)
}

#
# Function for computing the standard deviation of non-normally distributed data and its
# 95% (bootstrap) confidence interval
#
sdCI <- function(observations) {
  samplesd <- function(x, d) {return(sd(x[d]))}
  pointEstimate <- samplesd(observations)
  set.seed(0) # make deterministic
  bootstrap_samples <- boot(data = observations, statistic = samplesd, R = 5000)
  bootci <- boot.ci(bootstrap_samples, type = "bca")
  c(pointEstimate,  bootci$bca[4], bootci$bca[5])
}


#
# Internal function for rounding a number for textual presentation
#
format_number_ <- function(n, sigdigits, width = NA) {
  # I finally seem to have found a pretty formatting code that actually works!
  pad(formatC(signif(n, digits=sigdigits), digits=sigdigits, format="fg", flag="#", big.mark = " "), width)
}

#
# Pad a string with white spaces. Use NA for no padding and a negative width to pad right (align left).
#
pad <- function(s, width) {
  if (is.na(width))
    s
  else
    sprintf(paste('%', width, 's', sep=''), s)
    
}

#
# Round a number for textual presentation
# 
format_number <- function(n, sigdigits = NA, width = NA, percent = FALSE) {
  if (percent)
    n <- n * 100
  suffix <- ifelse(percent, '%', '')
  
  if (!is.na(sigdigits)) {
    s <- format_number_(n, width, sigdigits = sigdigits)
  } else {
    # When sigdigits = NA, we use what seems to be a reasonable manual heuristic for deciding on
    # the number of significant digits in our case (decoy study)
    if (abs(n) >= 10)
      s <- format_number_(n, sigdigits = 2, width = width)
    else if (abs(n) >= 2)
      s <- format_number_(n, sigdigits = 1, width = width)
    else if (abs(n) >= 1)
      s <- format_number_(n, sigdigits = 2, width = width)
    else
      s <- format_number_(n, sigdigits = 1, width = width)
  } 
  paste(s, suffix, sep='')
}

# Present a number and its CI
#
format_ci <- function(x, sigdigits = NA, width = NA, percent = FALSE) {
  nwidth <- floor((width - 8) / 3)
  pad(paste(
    format_number(x[1], sigdigits = sigdigits, width = nwidth, percent = percent),
    ', 95% CI [',
    format_number(x[2], sigdigits = sigdigits, width = nwidth, percent = percent),
    ', ',
    format_number(x[3], sigdigits = sigdigits, width = nwidth, percent = percent),
    ']',
    sep=''), width = width)
}

plotAllCI <- function(results, plotRange = c(-1, 1)) {
  palette <- c("#888888", "#EE4444", "#000000", "#4466FF")
  results$type <- as.factor(results$type)
  rows.to.plot <- 1:(nrow(results)) # plot all but the within-pair consistency score
  results.to.plot <- results[rows.to.plot,]
  # Simplify the ggplot visual theme
  theme_set(theme_bw() + theme(
    axis.title = element_blank(),
    axis.text.y = element_blank(),
    axis.ticks = element_blank(),
    panel.grid.major.y = element_blank(),
    panel.grid.minor.y = element_blank(),
    panel.grid.major.x = element_line(color = "#999999", size = 0.1),
    panel.grid.minor.x = element_blank(),
    panel.border = element_blank(),
    legend.key.size = unit(5,"mm"), 
    plot.margin=unit(c(0.1, 0.1, 0.1, 0.1),"cm"), #up, 
    text = element_text(size=17)))
  
  # Plot all confidence intervals
  p <- ggplot(data = results.to.plot, aes(x = rev(rows.to.plot), y = point, label=label)) +
    geom_pointrange(aes(ymin=ci.lower, ymax=ci.upper), size=1) +
    geom_point(size=5, colour="#ffffff", pch=21) +
    geom_text(color="#888888",y=plotRange[1], hjust=0, size=4.0) +
    geom_abline(intercept = 0, slope = 0, lty = 3, size=0.8) +
    #scale_y_continuous(limits = plotRange, labels = scales::percent, breaks=seq(-0.2, 0.6, 0.2)) +
    scale_y_continuous(limits = plotRange) +
    coord_flip()
  
}

plotAllCI_fixedBreaks <- function(results, plotRange = c(-1, 1), breaks) {
  palette <- c("#888888", "#EE4444", "#000000", "#4466FF")
  results$type <- as.factor(results$type)
  rows.to.plot <- 1:(nrow(results)) # plot all but the within-pair consistency score
  results.to.plot <- results[rows.to.plot,]
  # Simplify the ggplot visual theme
  theme_set(theme_bw() + theme(
    axis.title = element_blank(),
    axis.text.y = element_blank(),
    axis.ticks = element_blank(),
    panel.grid.major.y = element_blank(),
    panel.grid.minor.y = element_blank(),
    panel.grid.major.x = element_line(color = "#999999", size = 0.1),
    panel.grid.minor.x = element_blank(),
    panel.border = element_blank(),
    legend.key.size = unit(5,"mm"), 
    plot.margin=unit(c(0.1, 0.1, 0.1, 0.1),"cm"), #up, 
    text = element_text(size=17)))
  
  # Plot all confidence intervals
  p <- ggplot(data = results.to.plot, aes(x = rev(rows.to.plot), y = point, label=label)) +
    geom_pointrange(aes(ymin=ci.lower, ymax=ci.upper), size=1) +
    geom_point(size=5, colour="#ffffff", pch=21) +
    geom_text(color="#888888",y=plotRange[1], hjust=0, size=4.0) +
    geom_abline(intercept = 0, slope = 0, lty = 3, size=0.8) +
    scale_y_continuous(limits = plotRange,  breaks=breaks) +
    #scale_y_continuous(limits = plotRange) +
    coord_flip()
  
}



plotAllCI2 <- function(results, plotRange = c(0, 0.5)) {
  palette <- c("#888888", "#EE4444", "#000000", "#4466FF")
  results$type <- as.factor(results$type)
  rows.to.plot <- 1:(nrow(results)) # plot all but the within-pair consistency score
  results.to.plot <- results[rows.to.plot,]
  # Simplify the ggplot visual theme
  theme_set(theme_bw() + theme(
    axis.title = element_blank(),
    axis.text.y = element_blank(),
    axis.ticks = element_blank(),
    panel.grid.major.y = element_blank(),
    panel.grid.minor.y = element_blank(),
    panel.grid.major.x = element_line(color = "#999999", size = 0.1),
    panel.grid.minor.x = element_blank(),
    panel.border = element_blank(),
    legend.key.size = unit(5,"mm"), 
    plot.margin=unit(c(0.1, 0.1, 0.1, 0.1),"cm"), #up, 
    text = element_text(size=17)))
  
  # Plot all confidence intervals
  p <- ggplot(data = results.to.plot, aes(x = rev(rows.to.plot), y = point, label=label)) +
    geom_pointrange(aes(ymin=ci.lower, ymax=ci.upper), size=1) +
    geom_point(size=5, colour="#ffffff", pch=21) +
    geom_text(color="#888888",y=plotRange[1], hjust=0, size=4.0) +
    geom_abline(intercept = 0, slope = 0, lty = 3, size=0.8) +
    #scale_y_continuous(limits = plotRange, labels = scales::percent, breaks=seq(-0.2, 0.6, 0.2)) +
    scale_y_continuous(limits = plotRange) +
    
    coord_flip()
  
}



plotAllCI_TEST <- function(results, plotRange = c(-1.5, 1.5)) {
  palette <- c("#888888", "#EE4444", "#000000", "#4466FF")
  results$type <- as.factor(results$type)
  rows.to.plot <- 1:(nrow(results)) # plot all but the within-pair consistency score
  results.to.plot <- results[rows.to.plot,]
  # Simplify the ggplot visual theme
  theme_set(theme_bw() + theme(
    axis.title = element_blank(),
    axis.text.y = element_blank(),
    axis.ticks = element_blank(),
    panel.grid.major.y = element_blank(),
    panel.grid.minor.y = element_blank(),
    panel.grid.major.x = element_line(color = "#999999", size = 0.1),
    panel.grid.minor.x = element_blank(),
    panel.border = element_blank(),
    legend.key.size = unit(5,"mm"), 
    plot.margin=unit(c(0.1, 0.1, 0.1, 0.1),"cm"), #up, 
    text = element_text(size=17)))
  
  # Plot all confidence intervals
  p <- ggplot(data = results.to.plot, aes(x = rev(rows.to.plot), y = point, label=label)) +
    geom_pointrange(aes(ymin=ci.lower, ymax=ci.upper), size=1) +
    geom_point(size=0.9*5, colour="#ffffff", pch=21) +
    geom_text(color="#888888",y=plotRange[1], hjust=0, size=4.0) +
    geom_abline(intercept = 0, slope = 0, lty = 3, size=0.8) +
    #scale_y_continuous(limits = plotRange, labels = scales::percent, breaks=seq(-0.2, 0.6, 0.2)) +
    scale_y_continuous(limits = plotRange) +
    
    coord_flip()
  
}
